@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 import pytest
 from pytest import approx
 
@@ -37,7 +38,7 @@ def test_init():
         bh.Histogram(bh.axis.Integer(-1, 1), unknown_keyword="nh")
 
     h = bh.Histogram(bh.axis.Integer(-1, 2))
-    assert h.rank == 1
+    assert h.ndim == 1
     assert h.axes[0] == bh.axis.Integer(-1, 2)
     assert h.axes[0].extent == 5
     assert h.axes[0].size == 3
@@ -119,7 +120,6 @@ def test_fill_int_1d():
         h[-3]
 
 
-@pytest.mark.parametrize("flow", [True, False])
 def test_fill_1d(flow):
     h = bh.Histogram(bh.axis.Regular(3, -1, 2, underflow=flow, overflow=flow))
     with pytest.raises(ValueError):
@@ -180,10 +180,10 @@ def test_growth():
     h.fill(-1)
     h.fill(1)
     h.fill(1)
-    for i in range(255):
+    for _ in range(255):
         h.fill(0)
     h.fill(0)
-    for i in range(1000 - 256):
+    for _ in range(1000 - 256):
         h.fill(0)
     print(h.view(flow=True))
     assert h[bh.underflow] == 0
@@ -203,7 +203,52 @@ def test_growing_cats():
     assert h.size == 4
 
 
-@pytest.mark.parametrize("flow", [True, False])
+def test_metadata_add():
+    h1 = bh.Histogram(
+        bh.axis.IntCategory([1, 2, 3]), bh.axis.StrCategory(["1", "2", "3"])
+    )
+    h2 = bh.Histogram(
+        bh.axis.IntCategory([1, 2, 3]), bh.axis.StrCategory(["1", "2", "3"])
+    )
+    h1.fill([1, 2, 1, 2], ["1", "1", "2", "2"])
+    h2.fill([2, 3, 2, 3], ["2", "2", "3", "3"])
+
+    h3 = h1 + h2
+
+    assert h1.axes[0].size == 3
+    assert h2.axes[0].size == 3
+    assert h3.axes[0].size == 3
+
+    assert h1.axes[1].size == 3
+    assert h2.axes[1].size == 3
+    assert h3.axes[1].size == 3
+
+    assert h3[bh.loc(2), bh.loc("2")] == 2.0
+
+
+def test_grow_and_add():
+    h1 = bh.Histogram(
+        bh.axis.IntCategory([], growth=True), bh.axis.StrCategory([], growth=True)
+    )
+    h2 = bh.Histogram(
+        bh.axis.IntCategory([], growth=True), bh.axis.StrCategory([], growth=True)
+    )
+    h1.fill([1, 2, 1, 2], ["hi", "hi", "ho", "ho"])
+    h2.fill([2, 3, 4, 5], ["ho", "ho", "hum", "hum"])
+
+    h3 = h1 + h2
+
+    assert h1.axes[0].size == 2
+    assert h2.axes[0].size == 4
+    assert h3.axes[0].size == 5
+
+    assert h1.axes[1].size == 2
+    assert h2.axes[1].size == 2
+    assert h3.axes[1].size == 3
+
+    assert h3[bh.loc(2), bh.loc("ho")] == 2.0
+
+
 def test_fill_2d(flow):
     h = bh.Histogram(
         bh.axis.Integer(-1, 2, underflow=flow, overflow=flow),
@@ -237,7 +282,6 @@ def test_fill_2d(flow):
                 assert get(h, i, j) == m[i][j]
 
 
-@pytest.mark.parametrize("flow", [True, False])
 def test_add_2d(flow):
     h = bh.Histogram(
         bh.axis.Integer(-1, 2, underflow=flow, overflow=flow),
@@ -276,7 +320,6 @@ def test_add_2d_bad():
         a += b
 
 
-@pytest.mark.parametrize("flow", [True, False])
 def test_add_2d_w(flow):
     h = bh.Histogram(
         bh.axis.Integer(-1, 2, underflow=flow, overflow=flow),
@@ -430,6 +473,24 @@ def test_operators():
         h + h2
 
 
+def test_hist_hist_div():
+    h1 = bh.Histogram(bh.axis.Boolean())
+    h2 = bh.Histogram(bh.axis.Boolean())
+
+    h1[:] = (8, 6)
+    h2[:] = (2, 3)
+
+    h3 = h1 / h2
+
+    assert h3[False] == 4
+    assert h3[True] == 2
+
+    h1 /= h2
+
+    assert h1[False] == 4
+    assert h1[True] == 2
+
+
 def test_project():
     h = bh.Histogram(bh.axis.Integer(0, 2), bh.axis.Integer(1, 4))
     h.fill(0, 1)
@@ -437,12 +498,12 @@ def test_project():
     h.fill(1, 3)
 
     h0 = h.project(0)
-    assert h0.rank == 1
+    assert h0.ndim == 1
     assert h0.axes[0] == bh.axis.Integer(0, 2)
     assert [h0[i] for i in range(2)] == [2, 1]
 
     h1 = h.project(1)
-    assert h1.rank == 1
+    assert h1.ndim == 1
     assert h1.axes[0] == bh.axis.Integer(1, 4)
     assert [h1[i] for i in range(3)] == [1, 1, 1]
 
@@ -509,7 +570,7 @@ def test_pickle_0():
             a.fill(i, j, 0, 0, 0)
             for k in range(a.axes[2].extent):
                 a.fill(i, j, k, 0, 0)
-                for l in range(a.axes[3].extent):
+                for l in range(a.axes[3].extent):  # noqa: E741
                     a.fill(i, j, k, l, 0)
                     for m in range(a.axes[4].extent):
                         a.fill(i, j, k, l, m * 0.5 * np.pi)
@@ -518,7 +579,7 @@ def test_pickle_0():
     b = pickle.loads(io)
 
     assert id(a) != id(b)
-    assert a.rank == b.rank
+    assert a.ndim == b.ndim
     assert a.axes[0] == b.axes[0]
     assert a.axes[1] == b.axes[1]
     assert a.axes[2] == b.axes[2]
@@ -545,7 +606,7 @@ def test_pickle_1():
             a.fill(i, j, 0, 0, weight=10)
             for k in range(a.axes[2].extent):
                 a.fill(i, j, k, 0, weight=2)
-                for l in range(a.axes[3].extent):
+                for l in range(a.axes[3].extent):  # noqa: E741
                     a.fill(i, j, k, l, weight=5)
 
     io = BytesIO()
@@ -554,7 +615,7 @@ def test_pickle_1():
     b = pickle.load(io)
 
     assert id(a) != id(b)
-    assert a.rank == b.rank
+    assert a.ndim == b.ndim
     assert a.axes[0] == b.axes[0]
     assert a.axes[1] == b.axes[1]
     assert a.axes[2] == b.axes[2]
@@ -565,13 +626,41 @@ def test_pickle_1():
     assert a == b
 
 
+def test_pickle_bool():
+    a = bh.Histogram(bh.axis.Boolean(), bh.axis.Boolean(metadata={"one": 1}))
+    assert isinstance(a, bh.Histogram)
+
+    a.fill([True, True, False, False], [True, False, True, True])
+    a.fill([True, True, True], True)
+
+    assert a[True, True] == 4
+    assert a[True, False] == 1
+    assert a[False, True] == 2
+    assert a[False, False] == 0
+
+    io = BytesIO()
+    pickle.dump(a, io, protocol=-1)
+    io.seek(0)
+    b = pickle.load(io)
+
+    assert id(a) != id(b)
+    assert a.ndim == b.ndim
+    assert a.axes[0] == b.axes[0]
+    assert a.axes[1] == b.axes[1]
+    assert a.sum() == b.sum()
+    assert repr(a) == repr(b)
+    assert str(a) == str(b)
+    assert a == b
+    assert_array_equal(a.view(), b.view())
+
+
 # Numpy tests
 
 
 def test_numpy_conversion_0():
     a = bh.Histogram(bh.axis.Integer(0, 3, underflow=False, overflow=False))
     a.fill(0)
-    for i in range(5):
+    for _ in range(5):
         a.fill(1)
     c = np.array(a)  # a copy
     v = np.asarray(a)  # a view
@@ -580,13 +669,13 @@ def test_numpy_conversion_0():
         assert t.dtype == np.double  # CLASSIC: np.uint8
         assert_array_equal(t, (1, 5, 0))
 
-    for i in range(10):
+    for _ in range(10):
         a.fill(2)
     # copy does not change, but view does
     assert_array_equal(c, (1, 5, 0))
     assert_array_equal(v, (1, 5, 10))
 
-    for i in range(255):
+    for _ in range(255):
         a.fill(1)
     c = np.array(a)
 
@@ -599,7 +688,7 @@ def test_numpy_conversion_0():
 def test_numpy_conversion_1():
     # CLASSIC: was weight array
     h = bh.Histogram(bh.axis.Integer(0, 3))
-    for i in range(10):
+    for _ in range(10):
         h.fill(1, weight=3)
     c = np.array(h)  # a copy
     v = np.asarray(h)  # a view
@@ -618,7 +707,7 @@ def test_numpy_conversion_2():
     for i in range(a.axes[0].extent):
         for j in range(a.axes[1].extent):
             for k in range(a.axes[2].extent):
-                for m in range(i + j + k):
+                for _ in range(i + j + k):
                     a.fill(i, j, k)
                 r[i, j, k] = i + j + k
 
@@ -686,17 +775,17 @@ def test_numpy_conversion_5():
     )
 
     a.fill(0, 0)
-    for i in range(80):
+    for _ in range(80):
         a = a + a
     # a now holds a multiprecision type
     a.fill(1, 0)
-    for i in range(2):
+    for _ in range(2):
         a.fill(2, 0)
-    for i in range(3):
+    for _ in range(3):
         a.fill(0, 1)
-    for i in range(4):
+    for _ in range(4):
         a.fill(1, 1)
-    for i in range(5):
+    for _ in range(5):
         a.fill(2, 1)
     a1 = a.view()
     assert a1.shape == (3, 2)
